// Algo358F.cpp : Test driver for Algo358

//				Tests the CSVD routine, ACM Algorithm 358
//
//		Singular Value Decomposition, a = u * s * Conj(Tran(v)), a is destroyed by CSVD
//		the diagonal matrix s is output as a vector, m must be >= n
//		if smaller, a should be filled with zero rows
//		this code is adapted from Collected Algorithms from ACM, Algorithm 358
//		The transformation Conj(Tran(u)) is applied to the p vectors given in columns
//		n, n+1, ..., n+p-1 of matrix a
//		See: http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358.f
//		and: http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358_prb.f

#include "stdafx.h"
#include "Algo358.h"
#include "Global.h"
#define nL	"\n"		// new line
#define dL	"\n\n"		// double new line

using namespace System;
using namespace std;

void CSVD(complex<float> a[][maxA], int m, int n, int p, int nu, int nv,
		float *s, complex<float> u[][maxA], complex<float> v[][maxA]);
void TestCSVD();
void TimeStamp();
void WriteMatrix(complex<float> [][maxA], int, int);

int ti;		// test case number, 1 or 2
double time0;

using namespace MyNamespace;

// ------- initialize
void init(void)
{	
	String^ name = "Algo358F - Tests the CSVD routine, ACM Algorithm 358";
	String^ usage1 = "Usage: Algo358F [1, 2]";
	Global::DoubleSpace();
	Global::WriteLn(name);
	Global::WriteLn(Global::version);
	Global::WriteLn(Global::copyright);
	Global::WriteLn2(Global::trademark);
	Global::WriteLn2(usage1);
} // init

// The main entry point for the application Algo358F
int Algo358F(int in)
{
	init();
	ti =in;
	if (ti != 1 && ti != 2)
		ti = 1;
	Global::WriteLn2("Running test case " + ti + " of 2");
	time0 = Environment::TickCount / 1000.0;
	TimeStamp();
	Global::WriteLn(nL + "Algo358F:");
	Global::WriteLn2("Test the ACM TOMS 358 Algorithm CSVD");
	TestCSVD();
	Global::WriteLn("Algo358F:");
	Global::WriteLn2("Normal end of execution");
	TimeStamp();
	return 0;
} // Algo358F

void TestCSVD()		//test CSVD
{
	int i, j, k;
	int m = 5;
	int n = 5;
	if (ti == 2)
		n = 4;
	complex <float> a[maxA][maxA];
	float s[maxA];
	complex <float> u[maxA][maxA];
	complex <float> v[maxA][maxA];
	Global::WriteLn("TestCSVD");
	Global::WriteLn("Call ACM TOMS Algorithm 358 for the");
	Global::WriteLn("singular value decomposition:");
	Global::WriteLn("  A = U * S * V^H");
	Global::WriteLn("  of an M by N complex matrix A.");
	a[0][0] = complex<float>( 0.4499F, -0.1267F);	// Set the matrix
	a[0][1] = complex<float>(-0.2361F,  0.0775F);
	a[0][2] = complex<float>( 0.5008F, -0.7799F);
	a[0][3] = complex<float>(-0.7702F, -0.3143F);
	a[0][4] = complex<float>(-0.5084F,  0.5621F);
	a[1][0] = complex<float>(-0.8432F, -0.3443F);
	a[1][1] = complex<float>( 0.0186F, -0.6332F);
	a[1][2] = complex<float>( 0.3505F,  0.0166F);
	a[1][3] = complex<float>(-0.8892F,  0.2657F);
	a[1][4] = complex<float>(-0.5066F,  0.6005F);
	a[2][0] = complex<float>( 0.5896F,  0.2601F);
	a[2][1] = complex<float>( 0.8928F,  0.0103F);
	a[2][2] = complex<float>( 0.4350F, -0.2666F);
	a[2][3] = complex<float>(-0.7799F, -0.5512F);
	a[2][4] = complex<float>(-0.1022F, -0.4500F);
	a[3][0] = complex<float>( 0.3911F,  0.3234F);
	a[3][1] = complex<float>(-0.5605F,  0.7638F);
	a[3][2] = complex<float>(-0.2009F,  0.2707F);
	a[3][3] = complex<float>( 0.0314F, -0.4336F);
	a[3][4] = complex<float>(-0.1046F,  0.3267F);
	a[4][0] = complex<float>(-0.1395F, -0.1561F);
	a[4][1] = complex<float>( 0.3064F,  0.0263F);
	a[4][2] = complex<float>(-0.0975F,  0.9019F);
	a[4][3] = complex<float>( 0.2491F,  0.5787F);
	a[4][4] = complex<float>( 0.4106F, -0.8102F);
	Global::WriteLn2(nL + "Matrix A:");
	WriteMatrix(a, m, n);
	int p = 0;
	int nu = m;
	int nv = n;
	CSVD(a, m, n, p, nu, nv, s, u, v);
	Global::WriteLn2("Singular values (Diagonal of diagonal S matrix):");
	for (i = 0; i < n; i++)
	{
		Global::Write("  ");
		Global::Write(s[i].ToString("0.0000000"));
		Global::WriteLn();
	}
	if (m > n)
		Global::WriteLn2(nL + "U (Augmented to an orthogonal basis for the output space):");
	else
		Global::WriteLn2(nL + "U:");
	WriteMatrix(u, m, m);
	Global::WriteLn2("V:");
	WriteMatrix(v, n, n);
	for (i = 0; i < m; i++)		// Compute U * S * V^H
		 for (j = 0; j < n; j++)
		 {
				 a[i][j] = 0.0F, 0.0F;
				 for (k = 0; k < m && k < n; k++)
						 a[i][j] += u[i][k] * s[k] * conj(v[j][k]);
		 }
	Global::WriteLn2("Matrix U * S * V^H (Should equal the original A):");
	WriteMatrix(a, m, n);
} // TestCSVD

void WriteMatrix(complex <float> a[][maxA], int m, int n)
// print an m by n complex matrix
{
	for (int i = 0; i < m; i++)
	{
		for (int j = 0; j < n; j++)
		{
			Global::Write(" ");
			if (real(a[i][j]) >= 0.0F)
				Global::Write(" ");
			else
				Global::Write("-");
			Global::Write(abs(real(a[i][j])).ToString("0.0000"));
			if (imag(a[i][j]) >= 0.0F)
				Global::Write("+");
			else
				Global::Write("-");
			Global::Write(abs(imag(a[i][j])).ToString("0.0000"));
		}
		Global::WriteLn();
	}
	Global::WriteLn();
} // WriteMatrix

void TimeStamp()
// printout the current MM/dd/yyyy hh:mm:ss AM/PM  s.ss sec. as a timestamp
{
	double time1 = Environment::TickCount / 1000.0; // TickCount = number of milliseconds since Windows started.
	double timeD = time1 - time0;
	time0 = time1;
	DateTime dt = DateTime::Now;
	String^ st = dt.ToString("MM/dd/yyyy hh:mm:ss tt");
	Global::WriteLn(st + "  " + timeD.ToString("0.00") + " sec.");
} // TimeStamp

